library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


entity automate is
    port (
        I_clk         : in  std_logic;
        I_rst         : in  std_logic;
        I_clk_display : in  std_logic;
        I_button      : in  std_logic;
        I_invalide    : in  std_logic;
        I_end         : in  std_logic;
        O_counting    : out std_logic;
        O_store       : out std_logic;
        O_l_red       : out std_logic;
        O_l_green     : out std_logic
        );
end automate;

architecture a_automate of automate is

    type TYPE_ETAT is (
        st_wait_failed,
        st_wait_success,
        st_counting,
        st_compar,
        st_store,
        st_end_green,
        st_end_red
        );
    signal SR_STATE, SR_STATE_NEXT : TYPE_ETAT;

begin
    process (I_clk, I_rst)
    begin
        if(I_rst = '1')then
            SR_STATE <= st_wait_success;
        elsif rising_edge(I_clk)then
            SR_STATE <= SR_STATE_NEXT;
        end if;
    end process;
    
    process(SR_STATE, I_button, I_invalide, I_clk_display, I_end)
    begin          
        case SR_STATE is

            when st_wait_success =>
                O_counting      <= '0';
                O_store         <= '0';
                O_l_red         <= '0';
                O_l_green       <= '1';
                
                if I_button = '1' then
                    SR_STATE_NEXT <= st_counting;
                else
                    SR_STATE_NEXT <= st_wait_success;
                end if;
            
            when st_counting =>
                O_counting      <= '1';
                O_store         <= '0';
                O_l_red         <= '0';
                O_l_green       <= '0';
                
                if I_button = '0' then
                    SR_STATE_NEXT <= st_compar;
                else
                    SR_STATE_NEXT <= st_counting;
                end if;
            
            when st_compar =>
                O_counting      <= '0';
                O_store         <= '0';
                O_l_red         <= '0';
                O_l_green       <= '0';
                
                if I_invalide = '0' then
                    SR_STATE_NEXT <= st_store;
                else
                    SR_STATE_NEXT <= st_wait_failed;
                end if;
            
            when st_store =>
                O_counting      <= '0';
                O_store         <= '1';
                O_l_red         <= '0';
                O_l_green       <= '0';
                
                if I_end = '0' then
                    SR_STATE_NEXT <= st_wait_success;
                elsif I_end ='1' then
                    SR_STATE_NEXT <= st_end_red;
                end if;
            
            when st_wait_failed =>
                O_counting      <= '0';
                O_store         <= '0';
                O_l_red         <= '1';
                O_l_green       <= '0';
                
                if I_button = '1' then
                    SR_STATE_NEXT <= st_counting;
                else
                    SR_STATE_NEXT <= st_wait_failed;
                end if;
                
            when st_end_red =>
                O_counting      <= '0';
                O_store         <= '0';
                O_l_red         <= '1';
                O_l_green       <= '0';
                
                if I_clk_display = '1' then
                    SR_STATE_NEXT <= st_end_green;
                else
                    SR_STATE_NEXT <= st_end_red;
                end if;
                
            when st_end_green =>
                O_counting      <= '0';
                O_store         <= '0';
                O_l_red         <= '0';
                O_l_green       <= '1';
                
                if I_clk_display = '0' then
                    SR_STATE_NEXT <= st_end_red;
                else
                    SR_STATE_NEXT <= st_end_green;
                end if;

        end case;
    end process;


end a_automate;
